-- @ 50keda
-- @ 23.12.2016
-- @ v1.0
--
-- Script prevents user to trigger tipping if any fasten belts are mounted
--
-- You can use it and edit but keep the original author ;)
--
-- =======================================
--
-- @ PeterAH [MW]
-- @ 29.07.2020
-- @ v1.1
-- 
-- FS19 convert and some changes because of using simpleIC for door opening
-- instead of "trailerAnimateDoors.lua" in FS17

fastenBeltsChecker = {}

function fastenBeltsChecker.prerequisitesPresent(specializations)
    return (	
        SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations) and 
        SpecializationUtil.hasSpecialization(Trailer, specializations) and
        SpecializationUtil.hasSpecialization(TensionBelts, specializations)
    )
end

function fastenBeltsChecker.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", fastenBeltsChecker)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", fastenBeltsChecker)
end

function fastenBeltsChecker:onLoad(savegame)
	self.isAnyBeltFasten = fastenBeltsChecker.isAnyBeltFasten
	
	self.startTipping = Utils.appendedFunction(Trailer.startTipping, fastenBeltsChecker.startTipping)
	self.setTensionBeltsActive = Utils.appendedFunction(TensionBelts.setTensionBeltsActive, fastenBeltsChecker.setTensionBeltsActive)
    
	self.fbc_tipSides = {}
	local i = 0
    while true do
		local key = string.format("vehicle.fastenBeltsChecker.tipSide(%d)", i)
		if not hasXMLProperty(self.xmlFile, key) then
            break
        end
		self.fbc_tipSides[i+1] = getXMLBool(self.xmlFile, key.."#check")		
		i = i + 1
    end
    -- New in FS19:
    self.nodeLeftDoor = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.fastenBeltsChecker.leftDoor#node"), self.i3dMappings)
    self.nodeRightDoor = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.fastenBeltsChecker.rightDoor#node"), self.i3dMappings)
end

function fastenBeltsChecker:delete()
end

function fastenBeltsChecker:readStream(streamId, connection)
end

function fastenBeltsChecker:writeStream(streamId, connection)
end

function fastenBeltsChecker:mouseEvent(posX, posY, isDown, isUp, button)
end

function fastenBeltsChecker:keyEvent(unicode, sym, modifier, isDown)
end

function fastenBeltsChecker:onUpdate(dt)
    -- New in FS19:
    local _, _, zLeftDoor = getRotation(self.nodeLeftDoor)
    local _, _, zRightDoor = getRotation(self.nodeRightDoor)

    if zLeftDoor ~= 0 or zRightDoor ~= 0 then
        if self:isAnyBeltFasten() then
            self:setTensionBeltsActive(false, nil, true)
        end
    end
end

function fastenBeltsChecker:update(dt)
end

function fastenBeltsChecker:startTipping(tipSideIndex, noEventSend)
    -- local spec = self.spec_trailer

	if self:isAnyBeltFasten() then
        if self.fbc_tipSides[tipSideIndex] then
            -- FS17 stuff
            -- if self.isClient then
            --     g_currentMission:showBlinkingWarning(g_i18n:getText("warning_beltsAreFasten"))
            -- end
            -- as tipping process started already we have to cancel it, as it's forbidden to tip on this side with fasten belts
            --- spec.tipState = Trailer.TIPSTATE_CLOSED
            -- self:stopTipping()
            -- -- self:endTipping()

            -- remove belts when tipping, because stopTipping() is at FS19 not changigng the key state
            self:setTensionBeltsActive(false, nil, true)
        end
    end
end

function fastenBeltsChecker:setTensionBeltsActive(isActive, beltId, noEventSend)
    local spec = self.spec_trailer
	if isActive then
        if spec.tipState ~= Trailer.TIPSTATE_CLOSED then
            if self.fbc_tipSides[spec.currentTipSideIndex] then
                if self.isClient then
                    g_currentMission:showBlinkingWarning(g_i18n:getText("warning_fbc_tippingInProgress"))
                end
                -- immidiately remove all tension belts as we do not support belts during tipping on this side
                self:setTensionBeltsActive(false, nil, true)
            end
        end
        -- New in FS19:
        local _, _, zLeftDoor = getRotation(self.nodeLeftDoor)
        local _, _, zRightDoor = getRotation(self.nodeRightDoor)
        if zLeftDoor ~= 0 or zRightDoor ~= 0 then
            if self.isClient then
                g_currentMission:showBlinkingWarning(g_i18n:getText("warning_fbc_doorIsOpen"))
            end
            -- immidiately remove all tension belts as we do not support belts during a door is open
            self:setTensionBeltsActive(false, nil, true)
        end
    end
end

function fastenBeltsChecker:onDraw(isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
end

function fastenBeltsChecker:isAnyBeltFasten()
    local spec = self.spec_tensionBelts
	local fastenBelts = false
	if spec.tensionBelts ~= nil then
		for _, belt in pairs(spec.singleBelts) do
			if belt.mesh ~= nil then
				fastenBelts = true
				break
            end
        end
    end
	return fastenBelts
end
